#!/bin/bash

if [ $# != 2 ]; then
	echo "ERROR: landsat_import.sh usage: [input file(tar.gz)] [DEM layer]"
	exit 1
fi

if [ -z "$GISBASE" ] ; then
    echo "You must be in GRASS GIS to run this program." 1>&2
    exit 1
fi


if [ -z "$R_BIN" ]; then
	ENVFILE=`dirname $0`/landsat_setenv.txt
	echo $ENVFILE
	if [ ! -e $ENVFILE ]; then
		g.message -e "landsat_setenv.txt was not found. You must put landsat_setenv.txt in the same directory as this script."
		exit 1
	fi
	source $ENVFILE
fi


FILE=$1
DEM=$2

mkdir temp

#
# sun illumination terrain model
#
i.topo.corr -i base=$DEM zenith=$ZENITH azimuth=$AZIMUTH out="illumi"

tar xvfz $FILE -C temp
if [ $? -ne 0 ]; then
	exit 1
fi

cd temp
LIST=`ls *.TIF 2> /dev/null`
MET=`ls *.txt 2> /dev/null`

SELEV=`cat $MET | grep "SUN_ELEVATION " | cut -d "=" -f 2`
ZENITH=`echo "scale=8; 90-$SELEV" | bc`
AZIMUTH=`cat $MET | grep "SUN_AZIMUTH " | cut -d "=" -f 2`

echo "SOLAR_ZENITH_ANGLE = ${ZENITH} SOLAR_AZIMUTH_ANGLE=${AZIMUTH}"

for i in $LIST; do

	SUFFIX=`basename $i ".TIF" | cut -d '_' -f 2`
	if [ $SUFFIX = 'B8' ] || [ $SUFFIX = 'B10' ] || [ $SUFFIX = 'B11' ]; then
		continue
	fi

	if [ $SUFFIX = 'BQA' ]; then
		gdalwarp -of "GTiff" -r near -t_srs "`g.proj -j -f`" $i "temp.tif"
	else
		gdalwarp -of "GTiff" -r lanczos -t_srs "`g.proj -j -f`" $i "temp.tif"
	fi

	OUTPUTNAME=`basename $i ".TIF"`
	echo $i $OUTPUTNAME

	if [ $SUFFIX = 'BQA' ]; then
		r.in.gdal input="temp.tif" output=${OUTPUTNAME}
		if [ $? -ne 0 ]; then
			exit 1
		fi
	else
		r.in.gdal input="temp.tif" output=lo_${OUTPUTNAME}
		if [ $? -ne 0 ]; then
			exit 1
		fi
		r.colors map=lo_${OUTPUTNAME} color=grey
	fi

	rm -f "temp.tif"

done


#
# correct illumination.
#
PREFIX=`g.mlist type=rast pattern="lo_*" | head -n 1 | sed 's/[0-9]$//' | sed 's/^lo_//'`

#i.landsat.toar input_prefix=lo_${PREFIX} output_prefix=toar_ metfile=$MET

echo "convert DN to RADIANCE"
for I in `g.mlist type=rast pattern="lo_*"`; do

	BANDNO=`echo $I | cut -d "_" -f 3 | sed 's/B//'`
	REFMUL=`cat $MET | grep "REFLECTANCE_MULT_BAND_${BANDNO} " | cut -d "=" -f 2`
	REFADD=`cat $MET | grep "REFLECTANCE_ADD_BAND_${BANDNO} " | cut -d "=" -f 2`

	echo "$I REFLECTANCE_MULT = $REFMUL REFLECTANCE_ADD = $REFADD"
	echo "toar_${BANDNO} = (${REFMUL} * ${I} + (${REFADD})) / cos(${ZENITH})"
	r.mapcalc "toar_${BANDNO} = (${REFMUL} * ${I} + (${REFADD})) / cos(${ZENITH})"
	if [ $? -ne 0 ]; then
		exit 1
	fi

#	LMAX=`cat $MET | grep "RADIANCE_MAXIMUM_BAND_${BANDNO} " | cut -d "=" -f 2`
#	LMIN=`cat $MET | grep "RADIANCE_MINIMUM_BAND_${BANDNO} " | cut -d "=" -f 2`
#	QMAX=`cat $MET | grep "QUANTIZE_CAL_MAX_BAND_${BANDNO} " | cut -d "=" -f 2`
#	QMIN=`cat $MET | grep "QUANTIZE_CAL_MIN_BAND_${BANDNO} " | cut -d "=" -f 2`

#	echo "LMAX = ${LMAX} LMIN = ${LMIN} QMAX = ${QMAX} QMIN = ${QMIN}"

#	r.mapcalc "toar_${BANDNO} = (${I}-${QMIN})*(${LMAX}-${LMIN})/(${QMAX}-${QMIN})+(${LMIN})"

done

i.topo.corr base="illumi" input=`g.mlist type=rast pattern="toar_*" separator=comma` output="tcor_" zenith=$ZENITH method="c-factor"

#
# clean-up temp files and rename artifacts.
#

g.mremove -f rast="lo_*"
g.mremove -f rast="toar_*"
ALIST=`g.mlist type=rast pattern="tcor_.toar_*"`
for I in $ALIST; do
	g.rename rast=${I},`echo ${I} | sed "s/tcor_.toar_/${PREFIX}/"`
done

cd ..
rm -r -f temp
g.remove rast="illumi"
