#!/bin/bash


if [ -z "$GISBASE" ] ; then
    echo "ERROR: You must be in GRASS GIS to run this program." 1>&2
    exit 1
fi


if [ -z "$R_BIN" ]; then
	ENVFILE=`dirname $0`/landsat_setenv.txt
	echo $ENVFILE
	if [ ! -e $ENVFILE ]; then
		g.message -e "landsat_setenv.txt was not found. You must put landsat_setenv.txt in the same directory as this script."
		exit 1
	fi
	source $ENVFILE
fi

#
# specify target year.
#

echo "Specify target year "
read YEAR

echo $YEAR

#
# Choose image data to use.
#

echo "Select image data of the same year to use for interpolation."
echo "  1): use previous 48 days only."
echo "  2): use next 48 days only."
echo "  3): use both 48 days."
select DAY_TO_USE in 1 2 3; do
	case $DAY_TO_USE in
		1) break ;;
		2) break ;;
		3) break ;;
		*) echo "Invalid input" ;;
	esac
done

echo $DAY_TO_USE

#
# Confirmation of user input.
#
echo "---------- Your answer ------------\n"
echo " - Target year: $YEAR "
echo " - Image data to use:"
case $DAY_TO_USE in
	1) echo "  use previous 48 days only." ;;
	2) echo "  use next 48 days only." ;;
	3) echo "  use both 48 days." ;;
esac

while true; do 
	echo "Now it's ready to process. Execute (y/n) ?"
	read ANS

	if [ $ANS == "y" ]; then
		break
	elif [ $ANS == "n" ]; then
		echo "Quit script."
		exit 1
	else
		echo "Invalid input $ANS"
	fi
done


#
# check if images of target year are.
#
if [ `g.mlist type=rast pattern="${PRODUCT_NAME}*${YEAR}*" | wc -l` -eq 0 ]; then
	g.message -e "ERROR: no image layers of target year (${YEAR}) was found."
	exit 1
fi



# ------------------------------#

let "PREV_YEAR=$YEAR - 1"
let "NEXT_YEAR=$YEAR + 1"

#
# interpolate NULL cells of the target year's image by averaging adjacent 96 day's image.
#
function interpolateByBothDay(){

	for I in `g.mlist type=rast pattern="${PRODUCT_NAME}.*"`; do

		DATE=`echo $I | cut -d "_" -f 1 | cut -c 10-12`
		OUTPUT=interp_${I}
		g.copy rast=${I},${OUTPUT}

		echo $I

		P=16
		while [ $P -le 48 ]; do

			let "PREV=${DATE}-${P}"

			if [ $PREV -le 0 ]; then
				let "PREV=368+$PREV"
				PREV_TXT=`printf "%03d" $PREV`
				PREV_I=`echo $I | sed -e "s/${DATE}/${PREV_TXT}/" | sed -e "s/${YEAR}/${PREV_YEAR}/"`
			else
				PREV_TXT=`printf "%03d" $PREV`
				PREV_I=`echo $I | sed -e "s/${DATE}/${PREV_TXT}/"`
			fi

			eval `g.findfile element=cell file=$PREV_I`
			if [ ! "$file" ]; then
				echo "WARNING: Previous date image(${PREV_I}) not found." 1>&2
				let P="$P + 16"
				continue
			fi

			Q=16
			while [ $Q -le 48 ]; do

				let "NEXT=${DATE}+${Q}"

				if [ $NEXT -gt 361 ]; then
					let "NEXT=$NEXT-368"
					NEXT_TXT=`printf "%03d" $NEXT`
					NEXT_I=`echo $I | sed -e "s/${DATE}/${NEXT_TXT}/" | sed -e "s/${YEAR}/${NEXT_YEAR}/"`
				else
					NEXT_TXT=`printf "%03d" $NEXT`
					NEXT_I=`echo $I | sed -e "s/${DATE}/${NEXT_TXT}/"`
				fi

				eval `g.findfile element=cell file=$NEXT_I`
				if [ ! "$file" ]; then
					echo "WARNING: Next date image(${NEXT_I}) not found." 1>&2
					let Q="$Q + 16"
					continue
				fi

				let CP="$P / 16"
				let CQ="$Q / 16"
				let CR="$CP + $CQ"

				echo "target = ${I} prev = $PREV_I next = $NEXT_I"

				g.rename rast=${OUTPUT},"temp_imput"
				r.mapcalc "${OUTPUT}=isnull(temp_imput) ? (${PREV_I}*${CP}+${NEXT_I}*${CQ})/${CR} : temp_imput"
				if [ $? -ne 0 ]; then
					exit 1
				fi

				g.remove rast="temp_imput"
#				g.remove rast=${I}
#				g.rename rast=${OUTPUT},${I}

				let Q="$Q + 16"

			done

			let P="$P + 16"
		done

#		g.remove rast=${I}
#		g.rename rast=${OUTPUT},${I}

	done
}


#
# interpolate NULL cells of the target year's image by previous/next 48 day's image.
#
function interpolateByOnesideDay(){

	for I in `g.mlist type=rast pattern="${PRODUCT_NAME}.*"`; do

		DATE=`echo $I | cut -d "_" -f 1 | cut -c 10-12`
		OUTPUT=interp_${I}
		g.copy rast=${I},${OUTPUT}

		echo $I

		P=16
		while [ $P -le 48 ]; do
			if [ $DAY_TO_USE == 1 ]; then
				SRC=`expr $DATE - $P`
				if [ $SRC -le 0 ]; then
					let "SRC=368+$SRC"
					SRC_TXT=`printf "%03d" $SRC`
					SRC_I=`echo $I | sed -e "s/${DATE}/${SRC_TXT}/" | sed -e "s/${YEAR}/${PREV_YEAR}/"`
				else
					SRC_TXT=`printf "%03d" $SRC`
					SRC_I=`echo $I | sed -e "s/${DATE}/${SRC_TXT}/"`
				fi
			else
				SRC=`expr $DATE + $P`
				if [ $SRC -gt 361 ]; then
					let "SRC=$SRC-368"
					SRC_TXT=`printf "%03d" $SRC`
					SRC_I=`echo $I | sed -e "s/${DATE}/${SRC_TXT}/" | sed -e "s/${YEAR}/${NEXT_YEAR}/"`
				else
					SRC_TXT=`printf "%03d" $SRC`
					SRC_I=`echo $I | sed -e "s/${DATE}/${SRC_TXT}/"`
				fi
			fi

			eval `g.findfile element=cell file=$SRC_I`
			if [ ! "$file" ]; then
				echo "WARNING: Source date image(${SRC_I}) not found." 1>&2
				let P="$P + 16"
				continue
			fi

			echo "target = ${I} source = $SRC_I"

			g.rename rast=${OUTPUT},"temp_imput"
			r.mapcalc "${OUTPUT}=isnull(temp_imput) ? ${SRC_I} : temp_imput"
			if [ $? -ne 0 ]; then
				exit 1
			fi
			g.remove rast="temp_imput"

			let P="$P + 16"
		done

#		g.remove rast=${I}
#		g.rename rast=${OUTPUT},${I}
	done
}



#
# interpolate whole image.
#


if [ $DAY_TO_USE == 3 ]; then
	interpolateByBothDay
else
	interpolateByOnesideDay
fi

#for I in `g.mlist type=rast pattern="temp_rmc_*"`; do
#	g.rename ${I},`echo ${I} | sed -e "s/temp_rmc_/interp_/"`
#done
