#!/bin/bash

if [ $# != 4 ]; then
	echo "modis_check_classification usage: [classified raster layer] [truth data file] [output error matrix] [output KML]"
	exit 1
fi

if [ -z "$GISBASE" ] ; then
    echo "ERROR: You must be in GRASS GIS to run this program." 1>&2
    exit 1
fi

if [ -z "$R_BIN" ]; then
	ENVFILE=`dirname $0`/modis_setenv.txt
	echo $ENVFILE
	if [ ! -e $ENVFILE ]; then
		g.message -e "modis_setenv.txt was not found. You must put modis_setenv.txt in the same directory as this script."
		exit 1
	fi
	source $ENVFILE
fi

CLASSLAYER=$1
TRUTHDATA=$(cd $(dirname "$2") && pwd)/$(basename "$2")
OUTPUT=$3
KML=$4

#
# import truth data file.
#
TRUTHLAYER=`basename $TRUTHDATA | cut -d '.' -f 1`
v.in.ogr dsn="$TRUTHDATA" output=$TRUTHLAYER
if [ $? -ne 0 ]; then
	exit 1
fi

eval `g.findfile element=vector file=$TRUTHLAYER`
if [ -z $file ]; then
	g.message -e "import error $TRUTHLDATA."
	exit 1
fi

#
# check attribute of truth layer.
#
#COLUMN=`v.db.select map=$TRUTHLAYER column="class" where="1 = 2" 2>/dev/null`

CLASSLINE=`db.describe -c table=$TRUTHLAYER | grep -Gi "class"`
if [ -z "$CLASSLINE" ]; then
	g.message -e "column class not found."
	g.remove vect=$TRUTHLAYER
	exit 1
fi

CHECK=`echo $CLASSLINE | grep -Gi "integer"`
if [ -z "$CHECK" ]; then
	g.message -e "column class is not an INTEGER field."
	g.remove vect=$TRUTHLAYER
	exit 1
fi

COLUMN=`echo $CLASSLINE | cut -d " " -f 3 | cut -d ":" -f 1`

#
# check truth layer type.
#
eval `v.info -t map=$TRUTHLAYER`
if [ $areas -ne 0 ]; then
	echo "TRUTH layer is POLYGON."
	POLY=1
else
	echo "TRUTH layer is POINT."
	POLY=0
fi

#
# prepare truth layer.
#
v.db.renamecol map=$TRUTHLAYER column=$COLUMN,truth
if [ $? -ne 0 ]; then
	exit 1
fi
g.region vect=$TRUTHLAYER

#
# rasterize truth data.
#
v.to.rast --overwrite input=$TRUTHLAYER output=$TRUTHLAYER use=attr column=truth

#
# kappa analysis.
#
r.kappa -w classification=$CLASSLAYER reference=$TRUTHLAYER output=$OUTPUT

#
# if truth layer is POLYGON, vectorize raster layer as point.
#
if [ $POLY -eq 1 ]; then
	r.to.vect input=$TRUTHLAYER output=truth_temp feature=point
	g.remove rast=$TRUTHLAYER
	g.remove vect=$TRUTHLAYER
	v.db.renamecol map=truth_temp column=value,truth
	if [ $? -ne 0 ]; then
		exit 1
	fi
	TRUTHLAYER=truth_temp
else
	g.remove rast=$TRUTHLAYER
fi

#
# add classification column to truth vector.
#
v.db.addcol map=$TRUTHLAYER column="classified INTEGER"
if [ $? -ne 0 ]; then
	exit 1
fi
v.what.rast vector=$TRUTHLAYER raster=$CLASSLAYER column="classified"

#
# export truth vector to KML.
#
v.out.ogr input=$TRUTHLAYER type=point dsn=$KML format=KML

#
# clean-up temporary raster layer.
#

g.remove vect=$TRUTHLAYER
