#!/bin/bash

if [ $# != 3 ]; then
	echo "modis_classify_dtree usage: [training area vector] [image group name] [output filename (don't add extension)]"
	exit 1
fi

if [ -z "$GISBASE" ] ; then
    echo "ERROR: You must be in GRASS GIS to run this program." 1>&2
    exit 1
fi

if [ -z "$R_BIN" ]; then
	ENVFILE=`dirname $0`/modis_setenv.txt
	echo $ENVFILE
	if [ ! -e $ENVFILE ]; then
		g.message -e "modis_setenv.txt was not found. You must put modis_setenv.txt in the same directory as this script."
		exit 1
	fi
	source $ENVFILE
fi

eval `g.findfile element=vector file="$1"`
if [ ! "$file" ]; then
	g.message -e "Vector map <$1> not found"
	exit 1
fi

TRAINING=$1
OUTPUT=$(cd $(dirname "$3") && pwd)/$(basename "$3")
DATAOUTPUT=${OUTPUT}.data
NAMESOUTPUT=${OUTPUT}.names
TGROUP=$2

TEMPFILE="`g.tempfile pid=$$`"

TEMPDIR="tmp"


eval `g.findfile element=group file=$TGROUP`
if [ ! $file ]; then
	g.message -e "image group was not found [${TGROUP}]"
	exit 1
fi

#
# export training raster to XYZ.
#
eval `g.findfile element=CELL file=MASK`
if [ $file ]; then
	g.rename rast=MASK,_MASK
fi

mkdir $TEMPDIR
if [ $? -ne 0 ]; then
	g.message -e "Unable to create temporal directory"
	exit 1
fi
trap "rm -r -f tmp; exit 1" 1 2 3 15

v.to.rast --overwrite input=$TRAINING output=$TRAINING use=attr column=class
r.mask input=$TRAINING
r.out.xyz input=$TRAINING output="${TEMPDIR}/training.csv" fs=','
if [ $? -ne 0 ]; then
	exit 1
fi

IGROUP=`i.group -g -l group=$TGROUP 2> /dev/null`
printf "class.\n\nx: label.\ny: label.\n" > $NAMESOUTPUT
VAR=""
CLS=`r.category map=$TRAINING`
for C in $CLS; do
	VAR=${VAR},${C}
done
printf "class : %s.\n" `echo ${VAR} | sed 's/^,//'` >> $NAMESOUTPUT

for G in $IGROUP; do

	echo "-- $G"

	r.out.xyz input=${G} output="${TEMPDIR}/img_${G}.csv" fs=','
	if [ $? -ne 0 ]; then
		exit 1
	fi

	COLNAME=`echo $G | cut -d '@' -f 1`
	printf "%s: continuous.\n" $COLNAME >> $NAMESOUTPUT

done

$R_BIN --vanilla --slave << EOF

	# 3GBɐݒ
	memory.limit(size=3590)

	setwd("${TEMPDIR}")
	FLIST = list.files(getwd(), pattern="img_*")

	TABLE <- read.csv("training.csv", header=FALSE)
	colnames(TABLE)[3] <- "class"

	I <- 4
	for ( F in FLIST )
	{
		TEMP <- read.csv(F, header=FALSE)
		TABLE <- merge( TABLE, TEMP, all.y=TRUE )
		rm(TEMP)
		colnames(TABLE)[I] <- paste("band", I, sep="")
		I <- I + 1
	}

	TABLE_CC <- complete.cases(TABLE)
	TABLE <- TABLE[TABLE_CC,]
	write.table(TABLE, "table.data", sep=",", row.names=FALSE, col.names=FALSE)

EOF

cp ${TEMPDIR}/"table.data" ${DATAOUTPUT}

#
# restore MASK.
#
if [ `g.mlist pattern="_MASK"` ]; then
	r.mask -r
	g.rename rast=_MASK,MASK
fi

#
# clean-up temporary layer.
#
rm -r -f $TEMPDIR
